#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cassert>
#include "Neuron.h"

using namespace std;

Neuron::Neuron()
{
  this->Weight = NULL;
  this->CurrentState = 0.0f;
}

bool Neuron::Init(int WeigthCount)
{
  int i;

  assert(this->Weight == NULL);

  this->WeigthCount = WeigthCount;

  if(this->WeigthCount)
  {
    this->Weight = new double[WeigthCount];
    for(i = 0; i < WeigthCount; i++)
      this->Weight[i] = 1.0f;
  }

  return true;
}

Neuron::~Neuron()
{
  delete this->Weight;
}

NeuralNet::NeuralNet(int LayerCount, ...)
{
  int *p = &LayerCount + 1;
  int i, j;

  this->LayerCount = LayerCount;
  this->NeuronCountInLayer = new int[LayerCount];

  memcpy(this->NeuronCountInLayer, p, LayerCount * sizeof(int));

  this->Neurons = new Neuron*[LayerCount];
  for(i = 0; i < LayerCount; i++)
  {
    this->Neurons[i] = new Neuron[this->NeuronCountInLayer[i]];

    for(j = 0; j < this->NeuronCountInLayer[i]; j++)
      this->Neurons[i][j].Init(i == 0 ? 0 : this->NeuronCountInLayer[i - 1]);        
  }
}

NeuralNet::~NeuralNet()
{
  int i;

  assert(this->Neurons);
  assert(this->NeuronCountInLayer);

  for(i = 0; i < this->LayerCount; i++)
  {
    delete [] this->Neurons[i];
  }

  delete this->Neurons;
}

void 
NeuralNet::Go(double *DataIn, double *DataOut)
{
  int i, j, k;

  assert(this->Neurons);
  assert(this->NeuronCountInLayer);

  for(i = 0; i < this->NeuronCountInLayer[0]; i++)
    this->Neurons[0][i].CurrentState = DataIn[i];

  for(i = 1; i < this->LayerCount; i++)
  {
    for(j = 0; j < this->NeuronCountInLayer[i]; j++)
    {
      double Sum = 0.0f;
      Neuron *CurrentNeuron = &this->Neurons[i][j];
      Neuron *LastLayer = this->Neurons[i - 1];
      for(k = 0; k < this->NeuronCountInLayer[i - 1]; k++)
        Sum += CurrentNeuron->Weight[k] * LastLayer[k].CurrentState;

      CurrentNeuron->CurrentState = Sum; // F(Sum);
    }
  }

  for(i = 0; i < this->NeuronCountInLayer[this->LayerCount - 1]; i++)
    DataOut[i] = this->Neurons[LayerCount - 1][i].CurrentState;
}

void
NeuralNet::Dump()
{
  int i, j, k;

  for(i = 0; i < this->LayerCount; i++)
  {
    for(j = 0; j < this->NeuronCountInLayer[i]; j++)
    {
      printf("N[%i,%i] = %.3f  Weights: (", i, j, this->Neurons[i][j].CurrentState);
      if(i > 0)
      {
        for(k = 0; k < this->NeuronCountInLayer[i - 1]; k++)
        {
          printf("%.3f%s", this->Neurons[i][j].Weight[k], 
              k + 1 == this->NeuronCountInLayer[i - 1] ? "" : ", " );
        }
      }

      puts(")");
    }

    if(i + 1 < this->LayerCount)
      puts("--");
  }
}


void 
NeuralNet::ErrorCorrection(double CorrectionRatio, double *CorrectDataOut)
{
  int i, j, k;

  // XXX TO CHANGE
  for(i = this->LayerCount - 1; i >= this->LayerCount - 1; i--)
  {
    Neuron *CurrentLayer = this->Neurons[i];
 
    for(j = 0; j < this->NeuronCountInLayer[i]; j++)
    {
      double Error = - CurrentLayer[j].CurrentState;

      if(i == this->LayerCount - 1)
      {
        Error += CorrectDataOut[j];
      }
      else
      {
        // XXX TO CHANGE
      }

      for(k = 0; k < this->NeuronCountInLayer[i - 1]; k++)
      {
        CurrentLayer[j].Weight[k] += Error * CorrectionRatio * this->Neurons[i - 1][k].CurrentState;
      }      
    }    
  }

}

